/*
	PICTURE.CPP

	BeOS image file display routines, including the main function
	called from the engine:

		hugo_displaypicture

	for the Hugo Engine

	Copyright (c) 1995-2000 by Kent Tessman
*/

#if !defined (NO_GRAPHICS)

#include <DataIO.h>
#include <TranslationUtils.h>

#include "behugo.h"

extern "C"
{
#include "heheader.h"

int hugo_displaypicture(FILE *infile, long length);
}

BFile *TrytoOpenBFile(char *name, char *where);
int DisplayBBitmap(BBitmap *img);
int DisplayJPEG(BFile *file, long reslength);


int hugo_hasgraphics(void)
{
	return display_graphics;
}

/* hugo_displaypicture

	(This is the only routine that gets called from the engine proper.)

	Assumes that "filename" is hot--i.e., opened and positioned--but
	closes it before returning.

	Loads and displays a JPEG picture, sizing it to the screen area
	described by the current text window.  If smaller than the
	current text window, the image should be centered, not stretched.

	Ultimately, graphic formats other than JPEG may be supported,
	hence the identification of the type flag and the switch
	at the end.

	Returns false if it fails because of an ERROR.
*/

int hugo_displaypicture(FILE *infile, long reslength)
{
	long pos;
	BFile *file;
	
	/* So that graphics can be dynamically enabled/disabled */
	if (!hugo_hasgraphics())
	{
		fclose(infile);
		return true;		/* not an error */
	}

	// Reopen infile as a BFile at the same position
	pos = ftell(infile);
	fclose(infile);
	// If the resource is not blank, we're using a resource file,
	// so uppercase the name
	if (strcpy(loaded_resname, "")) strupr(loaded_filename);
	if (!(file = TrytoOpenBFile(loaded_filename, "games")))
	{
		if (!(file = TrytoOpenBFile(loaded_filename, "object")))
		{
			return false;
		}
	}
	// Seek BFile to the same position infile was at
	file->Seek((off_t)pos, SEEK_SET);	

	/* Before doing any drawing, mainly because we need to make sure there
	   is no scroll_offset, or anything like that:
	*/
	view->Update(false);

	switch (resource_type)
	{
		case JPEG_R:
			if (!DisplayJPEG(file, reslength)) goto Failed;
			break;

		default:        /* unrecognized */
#if defined (DEBUGGER)
			SwitchtoDebugger();
			DebugMessageBox("Picture Loading Error", "Unrecognized graphic format");
			SwitchtoGame();
#endif
			goto Failed;
	}

	delete file;
	return 1;	// success

Failed:
	delete file;
	return 0;
}


/* TrytoOpenBFile
 * 
 *   is similar to hemisc.c's TrytoOpen, but returns a BFile.
 */

extern char gamepath[];  // from hemisc.c

BFile *TrytoOpenBFile(char *name, char *where)
{
	char drive[MAXDRIVE], dir[MAXDIR], fname[MAXFILENAME], ext[MAXEXT];
	char envvar[32];
	BFile *tempfile;
	char temppath[MAXPATH];

	/* Try to open the given, vanilla filename */
	tempfile = new BFile(name, B_READ_ONLY);
	if (tempfile->InitCheck()==B_OK)
		return tempfile;
	delete tempfile;

	hugo_splitpath(name, drive, dir, fname, ext);  /* file to open */

	/* If the given filename doesn't already specify where to find it */
	if (!strcmp(drive, "") and !strcmp(dir, ""))
	{
		/* Check gamefile directory */
		hugo_makepath(temppath, "", gamepath, fname, ext);
		tempfile = new BFile(temppath, B_READ_ONLY);
		if (tempfile->InitCheck()==B_OK)
			return tempfile;
		delete tempfile;

		/* Check environment variables */
		strcpy(envvar, "hugo_");  /* make up the actual var. name */
		strcat(envvar, where);

		if (getenv(strupr(envvar)))
		{
			hugo_makepath(temppath, "", getenv(strupr(envvar)), fname, ext);

			tempfile = new BFile(temppath, B_READ_ONLY);
			if (tempfile->InitCheck()==B_OK)
				return tempfile;
			delete tempfile;
		}
	}

	/* return NULL if not openable */
	return NULL;
}


/* DisplayBBitmap
 * 
 *   is called by the completed image-loading routine (such as
 *   DisplayJPEG) in order to copy the loaded BBitmap to visible_view.
 */

int DisplayBBitmap(BBitmap *img)
{
	// First of all, find out if we're going to have to scale the
	// image down to fit in the defined window
	double scale = 1.0;
	BRect rect = BRect(img->Bounds());
	int width = (int)rect.right;
	int height = (int)rect.bottom;
	if (width > physical_windowwidth)
		scale = (double)physical_windowwidth/(double)width;
	if ((double)height*scale > (double)physical_windowheight)
		scale = (double)physical_windowheight/(double)height;
	if (scale!=1.0)
	{
		width = (int)((double)width*scale);
		height = (int)((double)height*scale);
	}

	// Figure out where the bitmap should be centered
	int x = (physical_windowwidth-width)/2;
	int y = (physical_windowheight-height)/2;

	rect.Set(physical_windowleft+x, physical_windowtop+y,
		physical_windowleft+x+width, physical_windowtop+y+height);

	if (bitmap->Lock())
	{
		view->DrawBitmap(img, rect);
		bitmap->Unlock();
	}
		
	return true;
}


/* DisplayJPEG */

int DisplayJPEG(BFile *file, long reslength)
{
	// Create a BMemoryIO object to hold the image data
	char *buffer;
	if (!(buffer = (char *)malloc(reslength*sizeof(char)))) return false;
	BMemoryIO memio(buffer, reslength);
	
	// Read the JPEG data into the buffer
	file->Read(buffer, reslength);
	
	// Call the translator to read the JPEG data into a BBitmap
	BBitmap *img = BTranslationUtils::GetBitmap(&memio);

	int result = false;
	if (img!=NULL)
	{
		result = DisplayBBitmap(img);
		delete img;
	}
	free(buffer);
	return result;
}

#else	// NO_GRAPHICS

extern "C"
{

#include "heheader.h"

int hugo_hasgraphics(void)
{
	return 0;
}

int hugo_displaypicture(FILE *infile)
{
	fclose(infile);
	return 1;	// not an error
}

}  // extern "C"

#endif  // NO_GRAPHICS
