/*
	SOUND.CPP

	BeOS routines for music (MOD, S3M, XM format modules) using MikMod
	and sound (RIFF WAVE samples):
	
		hugo_playmusic
		hugo_stopmusic
		hugo_playsample
		hugo_stopsample

	for the Hugo Engine

	Copyright (c) 1995-2000 by Kent Tessman

	Both hugo_playmusic() and hugo_playsample() are called with the
	file hot--i.e., opened and positioned to the start of the resource.
	It must be closed before returning.
	
	NOTE:  We use the MikMod library for sound output regardless since
	the software mixer takes care of handling looped (repeating) samples
	so we don't have to worry about it.
*/

extern "C"
{
#include "heheader.h"

int hugo_playmusic(FILE *infile, long reslength, char loop_flag);
void hugo_musicvolume(int vol);
void hugo_stopmusic(void);
int hugo_playsample(FILE *infile, long reslength, char loop_flag);
void hugo_samplevolume(int vol);
void hugo_stopsample(void);

// From heres.c:
long FindResource(char *filename, char *resname);
}

#if !defined (NO_SOUND)

#include <SoundPlayer.h>

#include "behugo.h"
#define MIKMODAPI
#include "mikmod.h"

#undef DEBUGGER

void SuspendAudio(void);
void ResumeAudio(void);

char audio_init_failed = true;
MODULE *modfile;
char music_is_playing = false;
int music_volume = 100, player_volume = 100;
SAMPLE *sample;
char sample_is_playing = false;
int sample_volume = 100;
int sample_channel = 0;
char audio_suspended = false;

#define MAXCHANNELS 32			/* for MikMod */


// The timer that will run the player:

thread_id mikmod_thread;

int32 TimerThread(void *data)
{
	while (true)
	{
		if (quit_he_thread) exit_thread(0);
		MikMod_Update();
		// 20 ms resolution
		snooze(20000);
	}
	return 0;
}


/* InitPlayer */

extern MDRIVER drv_be;

int InitPlayer(void)
{
	MikMod_RegisterLoader(&load_mod);	/* module loaders */
	MikMod_RegisterLoader(&load_s3m);
	MikMod_RegisterLoader(&load_xm);

//	MikMod_RegisterAllDrivers();		/* valid audio drivers */
	MikMod_RegisterDriver(&drv_be);

	md_mixfreq = 44100;			/* standard mixing frequency */
	md_mode = DMODE_16BITS | DMODE_STEREO |
		// software mixing
		DMODE_SOFT_MUSIC | DMODE_SOFT_SNDFX;
	md_device = 0;				/* standard device: autodetect */

	// This is 6 by default, but that's too much
	md_reverb = 0;

	if (MikMod_Init(""))			/* initialize driver */
	{
		audio_init_failed = true;
		return false;
	}
	audio_init_failed = false;

	/* Start the player here and keep it running globally */
	MikMod_SetNumVoices(MAXCHANNELS, MAXCHANNELS);
	MikMod_EnableOutput();
	// Start the timer
	mikmod_thread = spawn_thread(TimerThread, "Hugo Engine MikMod thread", B_NORMAL_PRIORITY, NULL);
	resume_thread(mikmod_thread);

	return true;
}

void PrintAudioError(void)
{
	static char printed_audio_error = 0;

	if (!printed_audio_error and enable_audio)
	{
		sprintf(line, "Unable to play sound/music:\n%s",
			MikMod_strerror(MikMod_errno));
		BAlert *alert = new BAlert("Sound Error", line,
			"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT);
		alert->Go();
		printed_audio_error = 1;
	}
}


/* hugo_playmusic

	Returns false if it fails because of an ERROR.
*/

int hugo_playmusic(FILE *f, long reslength, char loop_flag)
{
	if (audio_init_failed)
	{
		PrintAudioError();
		fclose(f);
		return true;	/* not an error */
	}

	if (music_is_playing)
	{
		hugo_stopmusic();
	}
	
	MikMod_Reset(NULL);

	// The file is already positioned
	modfile = Player_LoadFP(f, MAXCHANNELS, 0);
	fclose(f);

	if (modfile)
	{
		Player_Start(modfile);

		if (loop_flag)
			modfile->wrap = 1;
		else
			modfile->wrap = 0;

		if (audio_suspended)
			Player_SetVolume(0);
		else
			// Convert from 0-100
			Player_SetVolume(player_volume = music_volume*128/100);

		music_is_playing = true;
		
		return true;
	}
#if defined (DEBUGGER)
	else
		DebugMessageBox("Sound Library Error", MikMod_strerror(MikMod_errno));
#endif
	return false;
}


/* hugo_musicvolume */

void hugo_musicvolume(int vol)
{
	music_volume = vol;
}


/* hugo_stopmusic */

void hugo_stopmusic(void)
{
	if (audio_init_failed) return;

	if (music_is_playing)
	{
		music_is_playing = false;
		modfile->numpos = 0;
		
		Player_Free(modfile);
	}
}


/* hugo_playsample

	Returns false if it fails because of an ERROR.
*/

int hugo_playsample(FILE *f, long reslength, char loop_flag)
{
	int channel;

	if (audio_init_failed)
	{
		PrintAudioError();
		fclose(f);
		return true;	/* not an error */
	}

	if (sample_is_playing) hugo_stopsample();

	sample = Sample_LoadFP(f);
	fclose(f);

	if (sample)
	{
		/* Always play a sample on the last/highest channel */
		channel = MAXCHANNELS;

		// Convert volume from 0-100
		Voice_SetVolume(channel, 
			(!audio_suspended)?sample_volume*256/100:0);
		Voice_SetPanning(channel, 128);
		Voice_SetFrequency(channel, sample->speed);

		if (loop_flag)
		{
			sample->flags|=SF_LOOP;
			sample->loopstart = 0;
			sample->loopend = sample->length;
			Voice_Play(channel, sample, 0);
		}
		else
		{
			sample->flags&=~SF_LOOP;
			Voice_Play(channel, sample, 0);
		}

		sample_is_playing = true;
		
		return true;
	}
#if defined (DEBUGGER)
	else
		DebugMessageBox("Sound Library Error", MikMod_strerror(MikMod_errno));
#endif
	return false;
}


/* hugo_samplevolume */

void hugo_samplevolume(int vol)
{
	sample_volume = vol;
}


/* hugo_stopsample */

void hugo_stopsample(void)
{
	if (audio_init_failed) return;

	if (sample_is_playing)
	{
		sample_is_playing = false;
		sample->length = 0;
		sample->flags&=~SF_LOOP;

		Sample_Free(sample);
	}
}

int suspended_music_volume, suspended_sample_volume;

// From MikMod's drv_be.cpp:
extern BSoundPlayer *soco_bsbp;
float suspended_BSP_volume = 0;

void SuspendAudio(void)
{
	if (audio_suspended or audio_init_failed) return;

	if (sample_is_playing) Voice_SetVolume(MAXCHANNELS, 0);
	
	suspended_music_volume = player_volume;
	Player_SetVolume(player_volume = 0);
	if (!Player_Paused()) Player_TogglePause();
	
	suspended_BSP_volume = soco_bsbp->Volume();
	soco_bsbp->SetVolume(0);
	
	audio_suspended = true;
}

void ResumeAudio(void)
{
	if (!audio_suspended or audio_init_failed) return;

	if (sample_is_playing) Voice_SetVolume(MAXCHANNELS, sample_volume*256/100);

	player_volume = suspended_music_volume;
	Player_SetVolume(player_volume);
	if (Player_Paused()) Player_TogglePause();
	
	soco_bsbp->SetVolume(suspended_BSP_volume);
	
	audio_suspended = false;	
}

#else	/* NO_SOUND */

extern "C"
{

int hugo_playmusic(FILE *f, long reslength, char loop_flag)
{
	fclose(f);
	return true;	/* not an error */
}

void hugo_musicvolume(int vol)
{}

void hugo_stopmusic(void)
{}

int hugo_playsample(FILE *f, long reslength, char loop_flag)
{
	fclose(f);
	return true;	/* not an error */
}

void hugo_samplevolume(int vol)
{}

void hugo_stopsample(void)
{}

}	// extern "C"

#endif 	/* NO_SOUND */
